#version 150
uniform sampler2D sampler0;
uniform sampler2D sampler1;

uniform vec2 samplerSize1;

// object space to camera space transformation
uniform mat4 modelview_matrix;
 
// camera space to clip coordinates
uniform mat4 projection_matrix;

// texture space to camera space transformation
uniform mat4 texture_matrix;

uniform float max_displacement;
 
// incoming vertex position
in vec4 vertex_position;
// incoming vertex color
in vec4 vertex_color;
// incoming vertex texcoord
in vec4 vertex_texcoord0;
// incoming vertex texcoord
in vec4 vertex_texcoord1;

// eye space vertex coordinate
out vec4 vertexCoord;
// transformed vertex color
out vec4 color;
// eye space vertex normal
out vec3 normal;
// untransformed texture coordinate 0
out vec4 raw_texcoord0;
// transformed texture coordinate 0
out vec4 texcoord0;
// transformed texture coordinate 1
out vec4 texcoord1;
 
void main(void) {
	raw_texcoord0 = vertex_texcoord0;
	
	texcoord0 = ( texture_matrix * vertex_texcoord0 );
	texcoord1 = ( texture_matrix * vertex_texcoord1 );
	
	color = vertex_color;
	// transforming the incoming vertex position
	vertexCoord = vertex_position;
	vertexCoord.z += texture2DProj(sampler1, texcoord1).z*max_displacement;
	
	float dx = 1.0/samplerSize1.x;
	float dy = 1.0/samplerSize1.y;
	float u = texture2D(sampler1, vec2(texcoord1.x, texcoord1.y-dy)).r;
	float l = texture2D(sampler1, vec2(texcoord1.x-dx, texcoord1.y)).r;
	float r = texture2D(sampler1, vec2(texcoord1.x+dx, texcoord1.y)).r;
	float d = texture2D(sampler1, vec2(texcoord1.x, texcoord1.y+dy)).r;
	
	vec3 lon = (modelview_matrix * vec4(2.0*dx, 0.0, (r-l)*max_displacement, 0.0)).xyz;
	vec3 lat = (modelview_matrix * vec4(0.0, 2.0*dy, (d-u)*max_displacement, 0.0)).xyz; 
	
	normal = normalize(cross(lon, lat));
	
	vertexCoord = modelview_matrix * vertexCoord;
	gl_Position = projection_matrix * vertexCoord;
}