#version 150
uniform sampler2D sampler0;
uniform sampler2D sampler1;

uniform vec2 samplerSize1;

// object space to camera space transformation
uniform mat4 modelview_matrix;
 
// camera space to clip coordinates
uniform mat4 projection_matrix;

// texture space to camera space transformation
uniform mat4 texture_matrix;

uniform float max_displacement;

uniform vec3 sun;
uniform float shiny;
uniform float brightness;

uniform vec4 bobuck_position;
uniform vec4 drummer_position;
 
// incoming vertex position
in vec4 vertex_position;
// incoming vertex color
in vec4 vertex_color;
// incoming vertex texcoord
in vec4 vertex_texcoord0;
// incoming vertex texcoord
in vec4 vertex_texcoord1;

// eye space vertex coordinate
out vec4 vertexCoord;
// transformed vertex color
out vec4 color;
// eye space vertex normal
out vec3 normal;
// transformed texture coordinate 0
out vec4 texcoord0;
// transformed texture coordinate 1
out vec4 texcoord1;

// poopy (z)
out float poopy;

out float distance;
out float bobuck_distance;
out float drummer_distance;
 
void main(void) {
	texcoord0 = ( texture_matrix * vertex_texcoord0 );
	texcoord1 = ( texture_matrix * vertex_texcoord1 );
	
	color = vertex_color;
	// transforming the incoming vertex position
	vertexCoord = vertex_position;
	poopy = texture2DProj(sampler1, texcoord1).z;
	vertexCoord.z += poopy*max_displacement;
	
	float dx = 1.0/samplerSize1.x;
	float dy = 1.0/samplerSize1.y;
	float u = texture2D(sampler1, vec2(texcoord1.x, texcoord1.y-dy)).r;
	float l = texture2D(sampler1, vec2(texcoord1.x-dx, texcoord1.y)).r;
	float r = texture2D(sampler1, vec2(texcoord1.x+dx, texcoord1.y)).r;
	float d = texture2D(sampler1, vec2(texcoord1.x, texcoord1.y+dy)).r;
	
	vec3 lon = (modelview_matrix * vec4(2.0*dx, 0.0, (r-l)*max_displacement, 0.0)).xyz;
	vec3 lat = (modelview_matrix * vec4(0.0, 2.0*dy, (d-u)*max_displacement, 0.0)).xyz; 
	
	normal = normalize(cross(lon, lat));
	
	vertexCoord = modelview_matrix * vertexCoord;
	gl_Position = projection_matrix * vertexCoord;
	
	bobuck_distance = length(vertexCoord.xy - bobuck_position.xy);
	drummer_distance = length(vertexCoord.xy - drummer_position.xy);
	
	distance = length(vertexCoord);
	
	float multiplier = clamp(-dot(normal, sun), 0.25 + brightness, 1.0 + brightness);
	color.rgb *= vec3(multiplier) * vec3(1.0, 1.0, 1.0 + (smoothstep(1024.0, 4096.0, distance) * .1));
	
	// compute eye vector and normalize it 
	vec3 eye = normalize(-vertexCoord.xyz);
	// compute the half vector
	vec3 h = normalize(-sun + eye);

	// compute the specular term into spec
	float intSpec = max(dot(h,normal), 0.0);
	float spec = (0.5 + shiny) * pow(intSpec, 16.0);
	
	color.rgb += vec3(spec);
}