#version 150
uniform sampler2D sampler0;
uniform sampler2D sampler1;

// where the light is from or whatever
uniform vec2 lightSource;
uniform float distanceMultiplier;

// object space to camera space transformation
uniform mat4 modelview_matrix;
 
// camera space to clip coordinates
uniform mat4 projection_matrix;

// texture space to camera space transformation
uniform mat4 texture_matrix;
 
// incoming vertex position
in vec4 vertex_position;
// incoming vertex color
in vec4 vertex_color;
// incoming vertex normal
in vec3 vertex_normal;
// incoming vertex texcoord
in vec4 vertex_texcoord0;
// incoming vertex texcoord
in vec4 vertex_texcoord1;

// eye space vertex coordinate
out vec4 vertexCoord;
// transformed vertex color
out vec4 color;
// eye space vertex normal
out vec3 normal;
// transformed texture coordinate
out vec4 texcoord0;
// transformed texture coordinate
out vec4 texcoord1;

void main(void) {
	//not a proper transformation if modelview_matrix involves non-uniform scaling(TODO)
	normal = normalize(( modelview_matrix * vec4( vertex_normal, 0 ) ).xyz);
	
	texcoord0 = ( texture_matrix * vertex_texcoord0 );//
	texcoord1 = ( texture_matrix * vertex_texcoord1 );//
	
	color = vertex_color;
	// transforming the incoming vertex position
	float dist = length(vertex_position.xy - lightSource);
	dist *= dist * distanceMultiplier;
	vertexCoord = vertex_position;
	
	vertexCoord = modelview_matrix * vertexCoord;
	gl_Position = (projection_matrix * vertexCoord) * vec4(dist,dist,1.0,1.0);
}