#version 150
// object space to camera space transformation
uniform mat4 modelview_matrix;
 
// camera space to clip coordinates
uniform mat4 projection_matrix;

// texture space to camera space transformation
uniform mat4 texture_matrix;
 
// incoming vertex position
in vec4 vertex_position;
// incoming vertex color
in vec4 vertex_color;
// incoming vertex normal
in vec3 vertex_normal;
// incoming vertex texcoord
in vec4 vertex_texcoord0;

// eye space vertex coordinate
out vec4 vertexCoord;
// transformed vertex color
out vec4 color;
// eye space vertex normal
out vec3 normal;
// transformed texture coordinate
out vec4 texcoord0;

// the array of painting matrices and indices
uniform sampler2D sampler1;
uniform vec2 samplerSize1;

// number of paintings
uniform int count;
// ID offset
uniform int offset;

out float wComponent;
 
void main(void) {
	int w = int(samplerSize1.x);
	int id;
	{
		int i = count-gl_InstanceID;
		id = 4 * i;
		wComponent = float(offset + i);
	}
	
	int x = int(mod(float(id), float(w)));
	int y = id/w;
	
	vec4 a = texelFetch(sampler1, ivec2(x,y), 0);
	texcoord0 = ( texture_matrix * vertex_texcoord0 );
	texcoord0.z = a.w;
	a.w = 0.0;
	mat4 mvm = modelview_matrix * mat4(a,
							vec4(texelFetch(sampler1, ivec2(x+1, y), 0).xyz, 0.0),
							vec4(texelFetch(sampler1, ivec2(x+2, y), 0).xyz, 0.0),
							vec4(texelFetch(sampler1, ivec2(x+3, y), 0).xyz, 1.0));
							
	//not a proper transformation if modelview_matrix involves non-uniform scaling(TODO)
	normal = normalize(( mvm * vec4( vertex_normal, 0 ) ).xyz);
	
	color = vertex_color;
	// transforming the incoming vertex position
	vertexCoord = mvm * vertex_position;
	gl_Position = projection_matrix * vertexCoord;
}