#version 150
// object space to camera space transformation
uniform mat4 modelview_matrix;

// camera space to clip coordinates
uniform mat4 projection_matrix;

// the array of tulip coordinates
uniform sampler2D sampler1;

//bobuck position for z translate
uniform vec3 bobuck_position;

//trampler positions
uniform vec3 position1;
uniform vec3 position2;

//trampler strengths
uniform float trample1;
uniform float trample2;

// texture space to camera space transformation
uniform mat4 texture_matrix;

// tulip amount!!
uniform int tulip_amount;

// incoming vertex position
in vec4 vertex_position;
// incoming vertex color
in vec4 vertex_color;
// incoming vertex normal
in vec3 vertex_normal;
// incoming vertex texcoord
in vec4 vertex_texcoord0;
// incoming vertex texcoord
in vec4 vertex_texcoord1;

// eye space vertex coordinate
out vec4 vertexCoord;
// transformed vertex color
out vec4 color;
// eye space vertex normal
out vec3 normal;
// transformed texture coordinate
out vec4 texcoord0;
// transformed texture coordinate
out vec4 texcoord1;

void main(void) {
   vec3 pos = texelFetch(sampler1, ivec2(int(mod(float(gl_InstanceID), float(tulip_amount))), gl_InstanceID/tulip_amount), 0).xyz;
   
   vec3 offset = vec3(0.0);
   float len = 0.0;
   float trample_distance = 0.0;
   
   vec3 offset1 = pos - position1;
   float len1 = length(offset1);
   
   vec3 offset2 = pos - position2;
   float len2 = length(offset2);
   	
   	if (len1 < len2){
   		offset = offset1;
   		len = len1;
   		trample_distance = trample1;
   	}
   	else{
   		offset = offset2;
   		len = len2;
   		trample_distance = trample2;
   	}
   
   pos.z -= smoothstep(62.0, 70.0, length(pos.xy - bobuck_position.xy)) * 6.0;
   
   offset /= len;
   float trample = smoothstep(trample_distance, 0.0, len);
   float h = cos(trample);
   float v = sin(trample);
   vec3 u = vec3(offset.y, -offset.x, 0); //not renormalizing (no sick jumps)
   
   mat4 mvm = modelview_matrix * mat4(vec4(h + u.x * u.x * (1.0 - h), u.x * u.y * (1.0 - h) - u.z * v, u.x * u.z * (1.0 - h) + u.y * v, 0.0),
                           vec4(u.y * u.x * (1.0 - h) + u.z * v, h + u.y * u.y * (1.0 - h), u.y * u.z * (1.0 - h) - u.x * v, 0.0),
                           vec4(u.z * u.x * (1.0 - h) - u.y * v, u.z * u.y * (1.0 - h) + u.x * v, h + u.z * u.z * (1.0 - h), 0.0),
                           vec4(pos, 1.0));
                           
   //not a proper transformation if modelview_matrix involves non-uniform scaling(TODO)
   normal = normalize(( mvm * vec4( vertex_normal, 0.0 ) ).xyz);
   
   texcoord0 = ( texture_matrix * vertex_texcoord0 );//
   texcoord1 = ( texture_matrix * vertex_texcoord1 );//
   
   float inter = smoothstep(72.0, 0.0, len);
   color = vec4(.8 + (.2 * inter * vertex_color.r), .75 + (.25 * vertex_color.g * inter), .875 + (.125 * vertex_color.b * inter), 1.0);
   
   // transforming the incoming vertex position
   vertexCoord = mvm * vertex_position;
   gl_Position = projection_matrix * vertexCoord;
}