#version 150

uniform vec2 samplerSize0;

// object space to camera space transformation
uniform mat4 modelview_matrix;
 
// camera space to clip coordinates
uniform mat4 projection_matrix;

// texture space to camera space transformation
uniform mat4 texture_matrix;
 
// incoming vertex position
in vec4 vertex_position;
// incoming vertex texcoord
in vec4 vertex_texcoord0;

// transformed texture coordinate
out vec4[8] texcoord;

out vec2 tcc;

void main() 
{
  float dx = 1.0 / samplerSize0.x;
  float dy = 1.0 / samplerSize0.y;

  //     A1 B1 C1
  //  A0  A  B  C C4
  //  D0  D  E  F F4
  //  G0  G  H  I I4
  //     G5 H5 I5

  
  gl_Position = projection_matrix * (modelview_matrix * vertex_position);
  
  vec4 texMod = texture_matrix * vertex_texcoord0;
  tcc = texMod.xy;
  texcoord[0].xy = texMod.xy;
  texcoord[0].zw = vec2(0.0, 1.0);
  texcoord[1] = texMod.xxxy + vec4(    -dx, 0.0,  dx, -2.0*dy);  //  A1 B1 C1
  texcoord[2] = texMod.xxxy + vec4(    -dx, 0.0,  dx,     -dy);  //   A  B  C
  texcoord[3] = texMod.xxxy + vec4(    -dx, 0.0,  dx,       0);  //   D  E  F
  texcoord[4] = texMod.xxxy + vec4(    -dx, 0.0,  dx,      dy);  //   G  H  I
  texcoord[5] = texMod.xxxy + vec4(    -dx, 0.0,  dx,  2.0*dy);  //  G5 H5 I5
  texcoord[6] = texMod.xyyy + vec4(-2.0*dx, -dy, 0.0,      dy);  //  A0 D0 G0
  texcoord[7] = texMod.xyyy + vec4( 2.0*dx, -dy, 0.0,      dy);  //  C4 F4 I4 
}